import React from "react";
import PropTypes from "prop-types";
import styling from "@devcake/styling";
import { FormElement } from "../formElement";
import { Button } from "../../Button";
import { Icon } from "../../Icon";
import { Label } from "../Label";
import { ErrorMessage } from "../Error";
import { Color, ColorPropType, Type, TypePropType } from "../../types";
import { useStyle } from "../../Scope";

styling.setDefault("components.fileinput.buttons.select", "rounded-l-none");
styling.setDefault("components.fileinput.buttons.select.with-remove", "rounded-r-none");
styling.setDefault("components.fileinput.buttons.remove", "rounded-l-none border-l");
styling.link("components.fileinput.invalid", "components.form.invalid");
styling.link("components.fileinput.icons.select", "components.icons.document-add");
styling.link("components.fileinput.icons.remove", "components.icons.trash");

export type File = {
  name: string;
  data: Blob;
} | { name: null; data: null };
export type FileInputProps = FormElement<File | string> & {
  buttonType?: Type;
  buttonColor?: Color;
}

export const FileInput: React.FunctionComponent<FileInputProps> = ({
                                                                     buttonColor,
                                                                     buttonType,
                                                                     className,
                                                                     disabled,
                                                                     label,
                                                                     nullable,
                                                                     onChange,
                                                                     placeholder,
                                                                     validation,
                                                                     validationMessage,
                                                                     value
                                                                   }) => {
  const ref = React.createRef<HTMLInputElement>();
  const [file, setFile] = React.useState<File>({ name: null, data: null });
  const [error, setError] = React.useState(null);

  const validator = React.useMemo(() => validation instanceof RegExp ? v => validation.test(v) : validation, [validation]);

  const update = React.useCallback((evt: React.ChangeEvent<HTMLInputElement> | null) => {
    setError(null);

    let data = { name: null, data: null };
    if (evt != null) {
      const file = evt.target.files[0];
      if (file != undefined) {
        data = { name: file.name, data: file };
      }
    }

    if (data.name === null && !nullable) {
      setError("This field cannot be empty.");
    }

    if (validator) {
      const ret = validator(data.name);
      if (ret === false) setError(validationMessage);
      if (typeof ret === "string") setError(ret);
    }

    setFile(data);
    if (onChange) onChange(data);
  }, [setFile]);

  React.useEffect(() => {
    let name = typeof value === "object" ? value.name : value;
    setFile({ name, data: null });
  }, [value]);

  const [containerStyle, inputStyle, disabledStyle, invalidStyle] = useStyle(["input.container", "input", "input.disabled", "fileinput.invalid"], "components");
  const [iconSelect, iconRemove] = useStyle(["select", "remove"], "components.fileinput.icons");
  const [buttonSelect, buttonSelectWithRemove, buttonRemove] = useStyle(["select", "select.with-remove", "remove"], "components.fileinput.buttons");

  return <>
    {label && <Label className="mb-1">{label}</Label>}
    <div className={[
      "select-none flex flex-row",
      className,
      containerStyle,
      inputStyle,
      disabled ? disabledStyle : "",
      error ? invalidStyle : ""
    ].join(" ")}>
      <input
        type="file"
        className="hidden"
        ref={ref}
        onChange={update}
      />
      <span
        className={[
          (!file.name ? "text-gray-500" : ""),
          "px-3 py-2 flex-grow"
        ].join(" ")}
        onClick={() => ref.current.click()}
      >
        {file.name ?? placeholder}
      </span>
      <Button
        color={buttonColor ?? "indigo"}
        type={buttonType}
        className={[
          "flex-grow-0",
          buttonSelect,
          (file.name ? buttonSelectWithRemove : "")
        ].join(" ")}
        onClick={() => ref.current.click()}
      >
        <Icon
          icon={iconSelect}
          className="h-5"
        />
      </Button>
      {file.name && (
        <Button
          color={buttonColor ?? "red"}
          type={buttonType}
          className={[
            `flex-grow-0 border-${buttonColor ?? "red"}-300`,
            buttonRemove
          ].join(" ")}
          onClick={() => update(null)}
        >
          <Icon
            icon={iconRemove}
            className="h-5"
          />
        </Button>
      )}
    </div>
    {error && <ErrorMessage>{error}</ErrorMessage>}
  </>;
};

//@ts-ignore TS is complaining about nullable not being a boolean, somehow
FileInput.propTypes = {
  buttonColor: ColorPropType,
  buttonType: TypePropType,
  className: PropTypes.string,
  disabled: PropTypes.bool,
  label: PropTypes.node,
  nullable: PropTypes.bool,
  onChange: PropTypes.func,
  placeholder: PropTypes.string,
  value: PropTypes.string
};
FileInput.defaultProps = {
  buttonType: "secondary",
  disabled: false,
  nullable: true,
  onChange: () => null,
  placeholder: "Please select a file"
};
